package api

import (
	"fmt"

	"github.com/gin-gonic/gin"
)

// RegRouters 注册路由
func RegRouters(svc *Service) {
	root := svc.NewGinRouter("/")
	root.group.Use(Logger())

	// 用户登录
	root.POST("/user/login/password", UserLoginPassword)
	// 用户管理
	root.POSTJWT("/admin/user/create", CreateUser)
	root.POSTJWT("/admin/user/delete/:id", DeleteUser)
	root.GETJWT("/admin/user/list", UserList)
	root.GETJWT("/admin/user/detail/:id", UserDetail)
	root.POSTJWT("/admin/user/update", UpdateUser)

	// 获取自己的详情信息
	root.GETJWT("/user/profile", UserProfile)

	// 分类管理
	root.POSTJWT("/category/create", CreateCategory)
	root.POSTJWT("/category/delete/:id", DeleteCategory)
	root.GET("/category/list", CategoryList)
	root.GET("/category/parse/list", CategoryParseList)
	root.POSTJWT("/category/update", UpdateCategory)

	// 学习记录
	root.POSTJWT("/learnLog/create/:scope", CreateLearnLog)
	root.POSTJWT("/learnLog/sync/time/:id/:scope", SyncLearnTime)

	// 考核试题管理
	root.POSTJWT("/admin/test/create/:scope", CreateTest)
	root.POSTJWT("/admin/test/delete/:id/:scope", DeleteTest)
	root.GETJWT("/admin/test/list/:scope", TestList)
	root.GETJWT("/admin/test/detail/:id/:scope", TestDetail)
	root.POSTJWT("/admin/test/update/:scope", UpdateTest)
	root.POSTJWT("/admin/test/import/:scope", ImportTest)
	root.POSTJWT("/admin/test/export/:scope", ExportTest)
	root.POSTJWT("/admin/test/tmplate/:scope", TestTmplate)

	// 生成考核试题
	root.POSTJWT("/exeam/generate/:scope", GenerateExeam)

	// 提交考核记录
	root.POSTJWT("/exeamLog/submit/:scope", SubmitExeamLog)
	// 考核记录列表
	root.GETJWT("/exeamLog/list/:scope", ExeamLogList)

	// 用户id，可选：没有这个参数时是自己的信息，否则是指定id的信息
	// 本年每月学习时长 *type/year/uid
	root.GETJWT("/statistics/learn/process/:scope", StatisticsLearnProcess)
	// 个人:上次学习时长 uid
	root.GETJWT("/statistics/last/learnTime/:scope", StatisticsLastLearnTime)
	// 个人:用户总学习时长 uid
	root.GETJWT("/statistics/total/learnTime/:scope", StatisticsTotalLearnTime)
	// 个人: 获取最后一场考试的成绩 uid
	root.GETJWT("/statistics/last/exeamScore/:scope", StatisticsLastExeamScore)
	// 个人: 获取历史最好成绩 uid
	root.GETJWT("/statistics/best/exeamScore/:scope", StatisticsBestExeamScore)
	// 个人: 已学习模块数 uid
	root.GETJWT("/statistics/learned/modules/:scope", StatisticsLearnedModules)
	// 个人: 统计用户本年每月考核正确和错误数 year
	root.GETJWT("/statistics/exeamTF/rate/:scope", StatisticsExeamTFRate)

}

func Logger() gin.HandlerFunc {

	return func(c *gin.Context) {
		// 开始时间
		// start := time.Now()
		// 处理请求
		c.Next()
		// 结束时间
		// end := time.Now()
		//执行时间
		// latency := end.Sub(start)

		path := c.Request.URL.Path

		clientIP := c.ClientIP()
		// method := c.Request.Method
		// statusCode := c.Writer.Status()

		out := fmt.Sprintf(" =>%15s=> %s", clientIP, path)
		fmt.Println(out)
	}
}
